# -*- coding: utf-8 -*-
"""
Created on Tue Jun 18 11:34:58 2024

@author: bgregor
"""

# Adapted from the Numba Mandelbrot example:  https://numba.pydata.org/numba-doc/dev/user/examples.html



try:
    from matplotlib.pylab import imshow, show
    have_mpl = True
except ImportError:
    have_mpl = False
import numpy as np
from numba import njit
import functools
import time
import tqdm


def timer(func):
    ''' Takes a function as the argument.
        functools.wraps is a tool that helps properly
        handle doc strings and function names.'''
    @functools.wraps(func)
    def timing_wrapper(*args, **kwargs):
        # *args and **kwargs allow for variable numbers of
        # arguments
        start_t = time.perf_counter()
        # Call the function
        result = func(*args, **kwargs)
        end_t = time.perf_counter()
        print(f'{func.__name__}: {end_t - start_t:.4f} sec')
        return result
    return timing_wrapper



# First try running as-is.

# Then, uncomment the #@njit on the mandel function
# to numba-compile that.  How does the create_fractal 
# runtime improve?

# Next, for the mandel function use a caching decorator:
# @njit(cache=True)

#@njit
def mandel(x, y, max_iters):
    """
    Given the real and imaginary parts of a complex number,
    determine if it is a candidate for membership in the Mandelbrot
    set given a fixed number of iterations.
    """
    i = 0
    c = complex(x,y)
    z = 0.0j
    for i in range(max_iters):
        z = z * z + c
        if (z.real * z.real + z.imag * z.imag) >= 4:
            return i

    return 255


# Comment out the @timer if you want to try the Python profiler.
@timer  
#@njit
def create_fractal(min_x, max_x, min_y, max_y, image, iters):
    height = image.shape[0]
    width = image.shape[1]

    pixel_size_x = (max_x - min_x) / width
    pixel_size_y = (max_y - min_y) / height
    for x in tqdm.tqdm(range(width)):
        real = min_x + x * pixel_size_x
        for y in range(height):
            imag = min_y + y * pixel_size_y
            color = mandel(real, imag, iters)
            image[y, x] = color

    return image

image = np.zeros((500 * 2, 750 * 2), dtype=np.uint8)
create_fractal(-2.0, 1.0, -1.0, 1.0, image, 20)
 
if have_mpl:
    imshow(image)
    show()