# -*- coding: utf-8 -*-
"""
Created on Tue Oct  1 12:56:11 2019


An example for use with basic timing and profiling.


@author: bgregor
"""
import numpy as np
import time
import functools


# Pick a number to set the size of the matrix
mat_size = 10000
# Let's just multiply by 2.
scaling_value = 2.0


# Define a function decorator: this is a function
# that wraps another function and intercepts the incoming
# and return values.  Here this will provide timing info.

def timer(func):
    ''' Takes a function as the argument.
        functools.wraps is a tool that helps properly
        handle doc strings and function names.'''
    @functools.wraps(func)
    def timing_wrapper(*args, **kwargs):
        # *args and **kwargs allow for variable numbers of
        # arguments
        start_t = time.perf_counter()
        # Call the function
        result = func(*args, **kwargs)
        end_t = time.perf_counter()
        print(f'{func.__name__}: {end_t - start_t:.4f} sec')
        return result
    return timing_wrapper

# Apply the timer decorator to each function and
# call them normally...

@timer
def row_by_row(A,mat):
    ''' compute mat = A*mat row-by-row '''
    rows = mat.shape[0]
    for i in range(rows):
        mat[i,:] = A * mat[i,:] 
    return mat

@timer
def col_by_col(A,mat):
    ''' compute mat = A*mat col_by_col '''
    cols = mat.shape[1]
    for i in range(cols):
        mat[:,i] = A * mat[:,i] 
    return mat


@timer
def built_in(A,mat):
    ''' A*mat using built-in element-by-element'''
    mat[:,:] = A * mat[:,:]  
    return mat



if __name__=='__main__':
    # Create the matrix.
    mat = np.ones([mat_size,mat_size])
    # Run each function.
    mat = row_by_row(scaling_value, mat)
    mat = col_by_col(scaling_value, mat)
    mat = built_in(scaling_value, mat)


